@@ -20,7 +20,6 @@
 #include "common/perf_counters.h"
 #include "common/Thread.h"
 #include "common/ceph_context.h"
-#include "common/ceph_crypto.h"
 #include "common/config.h"
 #include "common/debug.h"
 #include "common/HeartbeatMap.h"
@@ -40,41 +39,6 @@
 
 using ceph::HeartbeatMap;
 
-namespace {
-
-class LockdepObs : public md_config_obs_t {
-public:
-  LockdepObs(CephContext *cct) : m_cct(cct), m_registered(false) {
-  }
-  virtual ~LockdepObs() {
-    if (m_registered) {
-      lockdep_unregister_ceph_context(m_cct);
-    }
-  }
-
-  const char** get_tracked_conf_keys() const {
-    static const char *KEYS[] = {"lockdep", NULL};
-    return KEYS;
-  }
-
-  void handle_conf_change(const md_config_t *conf,
-                          const std::set <std::string> &changed) {
-    if (conf->lockdep && !m_registered) {
-      lockdep_register_ceph_context(m_cct);
-      m_registered = true;
-    } else if (!conf->lockdep && m_registered) {
-      lockdep_unregister_ceph_context(m_cct);
-      m_registered = false;
-    }
-  }
-private:
-  CephContext *m_cct;
-  bool m_registered;
-};
-
-
-} // anonymous namespace
-
 class CephContextServiceThread : public Thread
 {
 public:
@@ -399,7 +363,6 @@ CephContext::CephContext(uint32_t module_type_)
     _conf(new md_config_t()),
     _log(NULL),
     _module_type(module_type_),
-    _crypto_inited(false),
     _service_thread(NULL),
     _log_obs(NULL),
     _admin_socket(NULL),
@@ -407,8 +370,7 @@ CephContext::CephContext(uint32_t module_type_)
     _perf_counters_conf_obs(NULL),
     _heartbeat_map(NULL),
     _crypto_none(NULL),
-    _crypto_aes(NULL),
-    _lockdep_obs(NULL)
+    _crypto_aes(NULL)
 {
   ceph_spin_init(&_service_thread_lock);
   ceph_spin_init(&_associated_objs_lock);
@@ -423,9 +385,6 @@ CephContext::CephContext(uint32_t module_type_)
   _cct_obs = new CephContextObs(this);
   _conf->add_observer(_cct_obs);
 
-  _lockdep_obs = new LockdepObs(this);
-  _conf->add_observer(_lockdep_obs);
-
   _perf_counters_collection = new PerfCountersCollection(this);
   _admin_socket = new AdminSocket(this);
   _heartbeat_map = new HeartbeatMap(this);
@@ -448,8 +407,8 @@ CephContext::CephContext(uint32_t module_type_)
   _admin_socket->register_command("log dump", "log dump", _admin_hook, "dump recent log entries to log file");
   _admin_socket->register_command("log reopen", "log reopen", _admin_hook, "reopen log file");
 
-  _crypto_none = new CryptoNone;
-  _crypto_aes = new CryptoAES;
+  _crypto_none = CryptoHandler::create(CEPH_CRYPTO_NONE);
+  _crypto_aes = CryptoHandler::create(CEPH_CRYPTO_AES);
 }
 
 CephContext::~CephContext()
@@ -460,6 +419,10 @@ CephContext::~CephContext()
        it != _associated_objs.end(); ++it)
     delete it->second;
 
+  if (_conf->lockdep) {
+    lockdep_unregister_ceph_context(this);
+  }
+
   _admin_socket->unregister_command("perfcounters_dump");
   _admin_socket->unregister_command("perf dump");
   _admin_socket->unregister_command("1");
@@ -493,10 +456,6 @@ CephContext::~CephContext()
   delete _cct_obs;
   _cct_obs = NULL;
 
-  _conf->remove_observer(_lockdep_obs);
-  delete _lockdep_obs;
-  _lockdep_obs = NULL;
-
   _log->stop();
   delete _log;
   _log = NULL;
@@ -508,14 +467,6 @@ CephContext::~CephContext()
 
   delete _crypto_none;
   delete _crypto_aes;
-  if (_crypto_inited)
-    ceph::crypto::shutdown();
-}
-
-void CephContext::init_crypto()
-{
-  ceph::crypto::init(this);
-  _crypto_inited = true;
 }
 
 void CephContext::start_service_thread()
