@@ -9,11 +9,11 @@
 #include "librbd/ObjectMap.h"
 #include "common/dout.h"
 #include "common/errno.h"
-#include <boost/lambda/bind.hpp> 
-#include <boost/lambda/construct.hpp>  
+#include <boost/lambda/bind.hpp>
+#include <boost/lambda/construct.hpp>
 
 #define dout_subsys ceph_subsys_rbd
-#undef dout_prefix 
+#undef dout_prefix
 #define dout_prefix *_dout << "librbd::AsyncFlattenRequest: "
 
 namespace librbd {
@@ -23,15 +23,15 @@ public:
   AsyncFlattenObjectContext(AsyncObjectThrottle &throttle, ImageCtx *image_ctx,
                             uint64_t object_size, ::SnapContext snapc,
                             uint64_t object_no)
-    : C_AsyncObjectThrottle(throttle), m_image_ctx(*image_ctx),
-      m_object_size(object_size), m_snapc(snapc), m_object_no(object_no)
+    : C_AsyncObjectThrottle(throttle, *image_ctx), m_object_size(object_size),
+      m_snapc(snapc), m_object_no(object_no)
   {
   }
 
   virtual int send() {
+    assert(m_image_ctx.owner_lock.is_locked());
     CephContext *cct = m_image_ctx.cct;
 
-    RWLock::RLocker l(m_image_ctx.owner_lock);
     if (m_image_ctx.image_watcher->is_lock_supported() &&
         !m_image_ctx.image_watcher->is_lock_owner()) {
       ldout(cct, 1) << "lost exclusive lock during flatten" << dendl;
@@ -49,13 +49,11 @@ public:
       return 1;
     }
 
-    int r = req->send();
-    assert(r == 0);
+    req->send();
     return 0;
   }
 
 private:
-  ImageCtx &m_image_ctx;
   uint64_t m_object_size;
   ::SnapContext m_snapc;
   uint64_t m_object_no;
@@ -91,6 +89,7 @@ bool AsyncFlattenRequest::should_complete(int r) {
 }
 
 void AsyncFlattenRequest::send() {
+  assert(m_image_ctx.owner_lock.is_locked());
   CephContext *cct = m_image_ctx.cct;
   ldout(cct, 5) << this << " send" << dendl;
 
@@ -100,91 +99,78 @@ void AsyncFlattenRequest::send() {
       boost::lambda::_1, &m_image_ctx, m_object_size, m_snapc,
       boost::lambda::_2));
   AsyncObjectThrottle *throttle = new AsyncObjectThrottle(
-    *this, context_factory, create_callback_context(), m_prog_ctx, 0,
-    m_overlap_objects);
-  throttle->start_ops(cct->_conf->rbd_concurrent_management_ops);
+    this, m_image_ctx, context_factory, create_callback_context(), &m_prog_ctx,
+    0, m_overlap_objects);
+  throttle->start_ops(m_image_ctx.concurrent_management_ops);
 }
 
 bool AsyncFlattenRequest::send_update_header() {
+  assert(m_image_ctx.owner_lock.is_locked());
   CephContext *cct = m_image_ctx.cct;
-  bool lost_exclusive_lock = false;
 
+  ldout(cct, 5) << this << " send_update_header" << dendl;
   m_state = STATE_UPDATE_HEADER;
+
+  // should have been canceled prior to releasing lock
+  assert(!m_image_ctx.image_watcher->is_lock_supported() ||
+         m_image_ctx.image_watcher->is_lock_owner());
+
   {
-    RWLock::RLocker l(m_image_ctx.owner_lock);
-    if (m_image_ctx.image_watcher->is_lock_supported() &&
-	!m_image_ctx.image_watcher->is_lock_owner()) {
-      ldout(cct, 1) << "lost exclusive lock during header update" << dendl;
-      lost_exclusive_lock = true;
-    } else {
-      ldout(cct, 5) << this << " send_update_header" << dendl;
-
-      RWLock::RLocker l2(m_image_ctx.parent_lock);
-      // stop early if the parent went away - it just means
-      // another flatten finished first, so this one is useless.
-      if (!m_image_ctx.parent) {
-	ldout(cct, 5) << "image already flattened" << dendl; 
-        return true;
-      }
-      m_ignore_enoent = true;
-      m_parent_spec = m_image_ctx.parent_md.spec;
-
-      // remove parent from this (base) image
-      librados::ObjectWriteOperation op;
-      if (m_image_ctx.image_watcher->is_lock_supported()) {
-        m_image_ctx.image_watcher->assert_header_locked(&op);
-      }
-      cls_client::remove_parent(&op);
-
-      librados::AioCompletion *rados_completion = create_callback_completion();
-      int r = m_image_ctx.md_ctx.aio_operate(m_image_ctx.header_oid,
-            				 rados_completion, &op);
-      assert(r == 0);
-      rados_completion->release();
+    RWLock::RLocker parent_locker(m_image_ctx.parent_lock);
+    // stop early if the parent went away - it just means
+    // another flatten finished first, so this one is useless.
+    if (!m_image_ctx.parent) {
+      ldout(cct, 5) << "image already flattened" << dendl;
+      return true;
     }
+    m_parent_spec = m_image_ctx.parent_md.spec;
   }
+  m_ignore_enoent = true;
 
-  if (lost_exclusive_lock) {
-    complete(-ERESTART);
+  // remove parent from this (base) image
+  librados::ObjectWriteOperation op;
+  if (m_image_ctx.image_watcher->is_lock_supported()) {
+    m_image_ctx.image_watcher->assert_header_locked(&op);
   }
+  cls_client::remove_parent(&op);
+
+  librados::AioCompletion *rados_completion = create_callback_completion();
+  int r = m_image_ctx.md_ctx.aio_operate(m_image_ctx.header_oid,
+        				 rados_completion, &op);
+  assert(r == 0);
+  rados_completion->release();
   return false;
 }
 
 bool AsyncFlattenRequest::send_update_children() {
   CephContext *cct = m_image_ctx.cct;
-  bool lost_exclusive_lock = false;
 
-  m_state = STATE_UPDATE_CHILDREN;
-  {
-    RWLock::RLocker l(m_image_ctx.owner_lock);
-    if (m_image_ctx.image_watcher->is_lock_supported() &&
-        !m_image_ctx.image_watcher->is_lock_owner()) {
-      ldout(cct, 1) << "lost exclusive lock during children update" << dendl;
-      lost_exclusive_lock = true;
-    } else {
-      // if there are no snaps, remove from the children object as well
-      // (if snapshots remain, they have their own parent info, and the child
-      // will be removed when the last snap goes away)
-      RWLock::RLocker l2(m_image_ctx.snap_lock);
-      if (!m_image_ctx.snaps.empty()) {
-        return true;
-      }
+  RWLock::RLocker owner_locker(m_image_ctx.owner_lock);
 
-      ldout(cct, 2) << "removing child from children list..." << dendl;
-      librados::ObjectWriteOperation op;
-      cls_client::remove_child(&op, m_parent_spec, m_image_ctx.id);
+  // should have been canceled prior to releasing lock
+  assert(!m_image_ctx.image_watcher->is_lock_supported() ||
+         m_image_ctx.image_watcher->is_lock_owner());
 
-      librados::AioCompletion *rados_completion = create_callback_completion();
-      int r = m_image_ctx.md_ctx.aio_operate(RBD_CHILDREN, rados_completion,
-					     &op);
-      assert(r == 0);
-      rados_completion->release();
-    }
-  }  
-
-  if (lost_exclusive_lock) {
-    complete(-ERESTART);
+  // if there are no snaps, remove from the children object as well
+  // (if snapshots remain, they have their own parent info, and the child
+  // will be removed when the last snap goes away)
+  RWLock::RLocker snap_locker(m_image_ctx.snap_lock);
+  if ((m_image_ctx.features & RBD_FEATURE_DEEP_FLATTEN) == 0 &&
+      !m_image_ctx.snaps.empty()) {
+    return true;
   }
+
+  ldout(cct, 2) << "removing child from children list..." << dendl;
+  m_state = STATE_UPDATE_CHILDREN;
+
+  librados::ObjectWriteOperation op;
+  cls_client::remove_child(&op, m_parent_spec, m_image_ctx.id);
+
+  librados::AioCompletion *rados_completion = create_callback_completion();
+  int r = m_image_ctx.md_ctx.aio_operate(RBD_CHILDREN, rados_completion,
+    				     &op);
+  assert(r == 0);
+  rados_completion->release();
   return false;
 }
 
