@@ -36,12 +36,14 @@
 #include "events/ETableClient.h"
 #include "events/ETableServer.h"
 
+#include "events/ENoOp.h"
+
 
 LogEvent *LogEvent::decode(bufferlist& bl)
 {
   // parse type, length
   bufferlist::iterator p = bl.begin();
-  __u32 type;
+  EventType type;
   LogEvent *event = NULL;
   ::decode(type, p);
 
@@ -62,7 +64,66 @@ LogEvent *LogEvent::decode(bufferlist& bl)
   return event;
 }
 
-LogEvent *LogEvent::decode_event(bufferlist& bl, bufferlist::iterator& p, __u32 type)
+
+std::string LogEvent::get_type_str() const
+{
+  switch(_type) {
+  case EVENT_SUBTREEMAP: return "SUBTREEMAP";
+  case EVENT_SUBTREEMAP_TEST: return "SUBTREEMAP_TEST";
+  case EVENT_EXPORT: return "EXPORT";
+  case EVENT_IMPORTSTART: return "IMPORTSTART";
+  case EVENT_IMPORTFINISH: return "IMPORTFINISH";
+  case EVENT_FRAGMENT: return "FRAGMENT";
+  case EVENT_RESETJOURNAL: return "RESETJOURNAL";
+  case EVENT_SESSION: return "SESSION";
+  case EVENT_SESSIONS_OLD: return "SESSIONS_OLD";
+  case EVENT_SESSIONS: return "SESSIONS";
+  case EVENT_UPDATE: return "UPDATE";
+  case EVENT_SLAVEUPDATE: return "SLAVEUPDATE";
+  case EVENT_OPEN: return "OPEN";
+  case EVENT_COMMITTED: return "COMMITTED";
+  case EVENT_TABLECLIENT: return "TABLECLIENT";
+  case EVENT_TABLESERVER: return "TABLESERVER";
+  case EVENT_NOOP: return "NOOP";
+
+  default:
+    generic_dout(0) << "get_type_str: unknown type " << _type << dendl;
+    return "UNKNOWN";
+  }
+}
+
+
+/*
+ * Resolve type string to type enum
+ *
+ * Return -1 if not found
+ */
+LogEvent::EventType LogEvent::str_to_type(std::string const &str)
+{
+  std::map<std::string, EventType> types;
+  types["SUBTREEMAP"] = EVENT_SUBTREEMAP;
+  types["SUBTREEMAP_TEST"] = EVENT_SUBTREEMAP_TEST;
+  types["EXPORT"] = EVENT_EXPORT;
+  types["IMPORTSTART"] = EVENT_IMPORTSTART;
+  types["IMPORTFINISH"] = EVENT_IMPORTFINISH;
+  types["FRAGMENT"] = EVENT_FRAGMENT;
+  types["RESETJOURNAL"] = EVENT_RESETJOURNAL;
+  types["SESSION"] = EVENT_SESSION;
+  types["SESSIONS_OLD"] = EVENT_SESSIONS_OLD;
+  types["SESSIONS"] = EVENT_SESSIONS;
+  types["UPDATE"] = EVENT_UPDATE;
+  types["SLAVEUPDATE"] = EVENT_SLAVEUPDATE;
+  types["OPEN"] = EVENT_OPEN;
+  types["COMMITTED"] = EVENT_COMMITTED;
+  types["TABLECLIENT"] = EVENT_TABLECLIENT;
+  types["TABLESERVER"] = EVENT_TABLESERVER;
+  types["NOOP"] = EVENT_NOOP;
+
+  return types[str];
+}
+
+
+LogEvent *LogEvent::decode_event(bufferlist& bl, bufferlist::iterator& p, LogEvent::EventType type)
 {
   int length = bl.length() - p.get_off();
   generic_dout(15) << "decode_log_event type " << type << ", size " << length << dendl;
@@ -94,6 +155,8 @@ LogEvent *LogEvent::decode_event(bufferlist& bl, bufferlist::iterator& p, __u32
   case EVENT_TABLECLIENT: le = new ETableClient; break;
   case EVENT_TABLESERVER: le = new ETableServer; break;
 
+  case EVENT_NOOP: le = new ENoOp; break;
+
   default:
     generic_dout(0) << "uh oh, unknown log event type " << type << " length " << length << dendl;
     return NULL;
