@@ -22,6 +22,7 @@
 #include "CInode.h"
 #include "CDentry.h"
 #include "CDir.h"
+#include "MDSContext.h"
 
 #include "include/unordered_set.h"
 using ceph::unordered_set;
@@ -32,9 +33,11 @@ class CDentry;
 class MDS;
 struct MDSlaveUpdate;
 
+typedef uint64_t log_segment_seq_t;
+
 class LogSegment {
  public:
-  const uint64_t seq;
+  const log_segment_seq_t seq;
   uint64_t offset, end;
   int num_events;
 
@@ -60,13 +63,24 @@ class LogSegment {
   // client request ids
   map<int, ceph_tid_t> last_client_tids;
 
+  // potentially dirty sessions
+  std::set<entity_name_t> touched_sessions;
+
   // table version
   version_t inotablev;
   version_t sessionmapv;
   map<int,version_t> tablev;
 
   // try to expire
-  void try_to_expire(MDS *mds, C_GatherBuilder &gather_bld, int op_prio);
+  void try_to_expire(MDS *mds, MDSGatherBuilder &gather_bld, int op_prio);
+
+  std::list<MDSInternalContextBase*> expiry_waiters;
+
+  void wait_for_expiry(MDSInternalContextBase *c)
+  {
+    assert(c != NULL);
+    expiry_waiters.push_back(c);
+  }
 
   // cons
   LogSegment(uint64_t _seq, loff_t off=-1) :
