@@ -418,6 +418,7 @@ int HashIndex::pre_split_folder(uint32_t pg_num, uint64_t expected_num_objs)
   // the below logic is inspired by rados.h#ceph_stable_mod,
   // it basically determines how many sub-folders should we
   // create for splitting
+  assert(pg_num_bits > 0); // otherwise BAD_SHIFT
   if (((1 << (pg_num_bits - 1)) | ps) >= pg_num) {
     ++split_bits;
   }
@@ -430,6 +431,7 @@ int HashIndex::pre_split_folder(uint32_t pg_num, uint64_t expected_num_objs)
     leavies = leavies >> 4;
   }
   for (uint32_t i = 0; i < subs; ++i) {
+    assert(split_bits <= 4); // otherwise BAD_SHIFT
     int v = tmp_id | (i << ((4 - split_bits) % 4));
     paths.push_back(to_hex(v));
     ret = create_path(paths);
@@ -476,10 +478,9 @@ int HashIndex::recursive_create_path(vector<string>& path, int level)
 {
   if (level == 0)
     return 0;
-  int ret;
   for (int i = 0; i < 16; ++i) {
     path.push_back(to_hex(i));
-    ret = create_path(path);
+    int ret = create_path(path);
     if (ret < 0 && ret != -EEXIST)
       return ret;
     ret = recursive_create_path(path, level - 1);
@@ -749,7 +750,7 @@ string HashIndex::get_hash_str(uint32_t hash) {
 
 string HashIndex::get_path_str(const ghobject_t &oid) {
   assert(!oid.is_max());
-  return get_hash_str(oid.hobj.hash);
+  return get_hash_str(oid.hobj.get_hash());
 }
 
 uint32_t HashIndex::hash_prefix_to_hash(string prefix) {
