@@ -2,7 +2,7 @@
 /*
  *	Copyright (C) 1991, NeXT Computer, Inc.  All Rights Reserverd.
  *
- *	File:	fsx.c
+ *	File:	fsx.cc
  *	Author:	Avadis Tevanian, Jr.
  *
  *	File system exerciser. 
@@ -482,7 +482,9 @@ __librbd_clone(struct rbd_ctx *ctx, const char *src_snapname,
 	uint64_t features = RBD_FEATURES_ALL;
 	if (krbd) {
 		features &= ~(RBD_FEATURE_EXCLUSIVE_LOCK |
-		              RBD_FEATURE_OBJECT_MAP);
+		              RBD_FEATURE_OBJECT_MAP     |
+                              RBD_FEATURE_FAST_DIFF      |
+                              RBD_FEATURE_DEEP_FLATTEN);
 	}
 	ret = rbd_clone2(ioctx, ctx->name, src_snapname, ioctx,
 			 dst_imagename, features, order,
@@ -628,8 +630,6 @@ krbd_write(struct rbd_ctx *ctx, uint64_t off, size_t len, const char *buf)
 int
 __krbd_flush(struct rbd_ctx *ctx)
 {
-	int ret;
-
 	if (o_direct)
 		return 0;
 
@@ -639,7 +639,7 @@ __krbd_flush(struct rbd_ctx *ctx)
 	 * lifetime of the block device and write directly to it.
 	 */
 	if (fsync(ctx->krbd_fd) < 0) {
-		ret = -errno;
+		int ret = -errno;
 		prt("fsync failed\n");
 		return ret;
 	}
@@ -659,6 +659,20 @@ krbd_discard(struct rbd_ctx *ctx, uint64_t off, uint64_t len)
 	uint64_t range[2] = { off, len };
 	int ret;
 
+	/*
+	 * BLKDISCARD doesn't affect dirty pages.  This means we can't
+	 * rely on discarded sectors to match good_buf (i.e. contain
+	 * zeros) without a preceding cache flush:
+	 *
+	 *   write 0..3M
+	 *   discard 1..2M
+	 *
+	 * results in "data data data" rather than "data 0000 data".
+	 */
+	ret = __krbd_flush(ctx);
+	if (ret < 0)
+		return ret;
+
 	/*
 	 * off and len must be 512-byte aligned, otherwise BLKDISCARD
 	 * will fail with -EINVAL.  This means that -K (enable krbd
@@ -677,10 +691,9 @@ int
 krbd_get_size(struct rbd_ctx *ctx, uint64_t *size)
 {
 	uint64_t bytes;
-	int ret;
 
 	if (ioctl(ctx->krbd_fd, BLKGETSIZE64, &bytes) < 0) {
-		ret = -errno;
+		int ret = -errno;
 		prt("BLKGETSIZE64 failed\n");
 		return ret;
 	}
@@ -698,11 +711,18 @@ krbd_resize(struct rbd_ctx *ctx, uint64_t size)
 	assert(size % truncbdy == 0);
 
 	/*
-	 * This is essential: when krbd detects a size change, it calls
-	 * revalidate_disk(), which ends up calling invalidate_bdev(),
-	 * which invalidates only clean buffers.  The cache flush makes
-	 * it invalidate everything, which is what we need if we are
-	 * shrinking.
+	 * When krbd detects a size change, it calls revalidate_disk(),
+	 * which ends up calling invalidate_bdev(), which invalidates
+	 * clean pages and does nothing about dirty pages beyond the
+	 * new size.  The preceding cache flush makes sure those pages
+	 * are invalidated, which is what we need on shrink:
+	 *
+	 *  write 0..1M
+	 *  resize 0
+	 *  resize 2M
+	 *  write 1..2M
+	 *
+	 * results in "data data" rather than "0000 data".
 	 */
 	ret = __krbd_flush(ctx);
 	if (ret < 0)
@@ -951,25 +971,22 @@ report_failure(int status)
 void
 check_buffers(char *good_buf, char *temp_buf, unsigned offset, unsigned size)
 {
-	unsigned char c, t;
-	unsigned i = 0;
-	unsigned n = 0;
-	unsigned op = 0;
-	unsigned bad = 0;
-
 	if (memcmp(good_buf + offset, temp_buf, size) != 0) {
+		unsigned i = 0;
+		unsigned n = 0;
+
 		prt("READ BAD DATA: offset = 0x%x, size = 0x%x, fname = %s\n",
 		    offset, size, iname);
 		prt("OFFSET\tGOOD\tBAD\tRANGE\n");
 		while (size > 0) {
-			c = good_buf[offset];
-			t = temp_buf[i];
+			unsigned char c = good_buf[offset];
+			unsigned char t = temp_buf[i];
 			if (c != t) {
 			        if (n < 16) {
-					bad = short_at(&temp_buf[i]);
+					unsigned bad = short_at(&temp_buf[i]);
 				        prt("0x%5x\t0x%04x\t0x%04x", offset,
 				            short_at(&good_buf[offset]), bad);
-					op = temp_buf[offset & 1 ? i+1 : i];
+					unsigned op = temp_buf[offset & 1 ? i+1 : i];
 				        prt("\t0x%5x\n", n);
 					if (op)
 						prt("operation# (mod 256) for "
@@ -1979,7 +1996,7 @@ main(int argc, char **argv)
 		case 'b':
 			simulatedopcount = getnum(optarg, &endp);
 			if (!quiet)
-				fprintf(stdout, "Will begin at operation %ld\n",
+				fprintf(stdout, "Will begin at operation %lu\n",
 					simulatedopcount);
 			if (simulatedopcount == 0)
 				usage();
@@ -2006,9 +2023,12 @@ main(int argc, char **argv)
 				usage();
 			break;
 		case 'l':
-			maxfilelen = getnum(optarg, &endp);
-			if (maxfilelen <= 0)
-				usage();
+			{
+				int _num = getnum(optarg, &endp);
+				if (_num <= 0)
+					usage();
+				maxfilelen = _num;
+			}
 			break;
 		case 'm':
 			monitorstart = getnum(optarg, &endp);
@@ -2109,7 +2129,8 @@ main(int argc, char **argv)
 				prt("file name to long\n");
 				exit(1);
 			}
-			strncpy(logfile, dirpath, sizeof(logfile));
+			strncpy(logfile, dirpath, sizeof(logfile)-1);
+			logfile[sizeof(logfile)-1] = '\0';
 			if (strlen(logfile) < sizeof(logfile)-2) {
 				strcat(logfile, "/");
 			} else {
